@@ -6,6 +6,10 @@ from .utils import upload_s3, download_s3, upload_artifact, download_artifact
 from collections import UserDict, UserList
 
 class ArgoObjectDict(UserDict):
+    """
+    Generate ArgoObjectDict and ArgoObjectList on initialization rather than on __getattr__,
+    otherwise modify a.b.c will not take effect
+    """
     def __init__(self, d):
         super().__init__(d)
         for key, value in self.items():
@@ -29,6 +33,9 @@ class ArgoObjectDict(UserDict):
 
         self.data[key] = value
 
+    def recover(self):
+        return {key: value.recover() if isinstance(value, (ArgoObjectDict, ArgoObjectList)) else value for key, value in self.data.items()}
+
 class ArgoObjectList(UserList):
     def __init__(self, l):
         super().__init__(l)
@@ -38,6 +45,9 @@ class ArgoObjectList(UserList):
             elif isinstance(value, list):
                 self.data[i] = ArgoObjectList(value)
 
+    def recover(self):
+        return [value.recover() if isinstance(value, (ArgoObjectDict, ArgoObjectList)) else value for value in self.data]
+
 class ArgoStep(ArgoObjectDict):
     def __init__(self, step):
         super().__init__(step)
@@ -57,7 +67,7 @@ class ArgoStep(ArgoObjectDict):
                 parameters[par.name] = par
                 if hasattr(par, "description") and par.description is not None:
                     desc = jsonpickle.loads(par.description)
-                    if desc["type"] != str:
+                    if desc["type"] != str(str):
                         parameters[par.name].value = jsonpickle.loads(par.value)
             io.parameters = parameters
 
@@ -74,14 +84,17 @@ class ArgoStep(ArgoObjectDict):
                         io.parameters = {}
                     with tempfile.TemporaryDirectory() as tmpdir:
                         download_artifact(art, path=tmpdir)
-                        with open(os.path.join(tmpdir, name[13:]), "r") as f:
+                        fs = os.listdir(tmpdir)
+                        assert len(fs) == 1
+                        with open(os.path.join(tmpdir, fs[0]), "r") as f:
                             content = jsonpickle.loads(f.read())
                             param = {"name": name[13:], "save_as_artifact": True}
-                            if "type" in content and content["type"] == str:
+                            if "type" in content:
                                 param["type"] = content["type"]
-                                param["value"] = content["value"]
-                            else:
+                            if "type" in content and content["type"] != str(str):
                                 param["value"] = jsonpickle.loads(content["value"])
+                            else:
+                                param["value"] = content["value"]
                             io.parameters[name[13:]] = ArgoObjectDict(param)
 
     def modify_output_parameter(self, name, value):
