@@ -6,11 +6,10 @@ from .opio import Artifact, OPIO, OPIOSign
 class OP(ABC):
     """
     Python class OP
-    :param progress_total: an int representing total progress
-    :param progress_current: an int representing currenet progress
-    :method get_input_sign: get the signature of the inputs
-    :method get_output_sign: get the signature of the outputs
-    :method execute: execution of the OP
+
+    Args:
+        progress_total: an int representing total progress
+        progress_current: an int representing currenet progress
     """
     progress_total = 1
     progress_current = 0
